@@ -27,14 +27,8 @@
  *
  */
 
-#import "IOFireWireUserClient.h"
-#import "IOFireWireLink.h"
-#import "IOFireWireNub.h"
-#import "IOFireWireController.h"
-#import "IOConfigDirectory.h"
-
-#import <IOKit/assert.h>
-#import <IOKit/IOMessage.h>
+// public
+#import <IOKit/firewire/IOFireWireNub.h>
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -45,6 +39,19 @@ OSMetaClassDefineReservedUnused(IOFireWireNub, 1);
 OSMetaClassDefineReservedUnused(IOFireWireNub, 2);
 OSMetaClassDefineReservedUnused(IOFireWireNub, 3);
 
+#import <IOKit/firewire/IOFireWireController.h>
+#import <IOKit/firewire/IOConfigDirectory.h>
+
+// protected
+#import <IOKit/firewire/IOFireWireLink.h>
+
+// private
+#import "IOFireWireUserClient.h"
+
+// system
+#import <IOKit/assert.h>
+#import <IOKit/IOMessage.h>
+
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
 bool IOFireWireNub::init(OSDictionary * propTable)
@@ -72,6 +79,9 @@ void IOFireWireNub::free()
 	if( fConfigDirectorySet )
 		fConfigDirectorySet->release();
 		
+    if(fControl)
+        fControl->release();
+
     IOService::free();
 }
 
